; =========================================
; Lichteffekte fr Flugmodelle mit ATtiny13
; =========================================
;
; Version 1.00 (03.06.2008)
;
; Fuse-Bits:	BODLEVEL1=0	\ BrownOut-Spannung
;		BODLEVEL0=0	/ =4,3V
;		CKSEL1=1	\ Interner Oszillator
;		CKSEL0=0	/ =9,6MHz
;		CKDIV8=1	- Vorteiler /8 aus
;		SUT1=1		\ Maximale
;		SUT0=0		/ Startverzgerung
;
; Ports:	PB0 - Ausgang 20mA (L-aktiv)	LED links/rechts/hinten
;		PB1 - Ausgang 20mA (L-aktiv)	LED vorn
;		PB2 - Ausgang 10mA (L-aktiv)	LED Blitz 1
;		PB3 - Eingang (Pull-up)		Eingang vom Empfnger
;		PB4 - Ausgang 10mA (L-aktiv)	LED Blitz 2
;		PB5 - Eingang (Pull-up)		Reset
;
; Timer:	Timer0, wird mit Vorteiler 64 betrieben, Taktfrequenz=
;		150kHz, Interrupt bei Overflow und Zhlung eines H-Byte
;
.nolist
.include "tn13def.inc"
.list
;
;
; ---------------------------------------------------------------------
;
; Folgende 3 Beleuchtungseffekte knnen einzeln aktiviert werden:
;
; Effekt 1 - Positionslichter - 3 LEDs (links/rechts/hinten)
; Effekt 2 - Blitzlichter - 2 LEDs
; Effekt 3 - Scheinwerfer - 1 LED
;
; Steht am Sender ein Poti fr die Beleuchtung zur Verfgung, dann kann
; mit den folgenden 3 Werten die Poti-Stellung fr jeden Effekt im
; Bereich von -100% bis +100% individuell festgelegt werden. -100% ent-
; spricht dem linken Anschlag, 0% entspricht der Mittelstellung und
; +100% entspricht dem rechten Anschlag:
;
.equ	effct1=	-50	;Effekt 1 bei Stellung -50% aktivieren
.equ	effct2=	0	;Effekt 2 bei Stellung 0% aktivieren
.equ	effct3=	50	;Effekt 3 bei Stellung +50% aktivieren
;
; ---------------------------------------------------------------------
;
; Die folgenden Werte nicht ndern, sie werden von den oben eingegebe-
; nen Werten abgeleitet und in den Zhlerstand von Timer0 umgerechnet.
;
.equ	impcn1=	((effct1+100)*5+1000)*100/667 ;Zhlerstand fr Effekt 1
.equ	impcn2=	((effct2+100)*5+1000)*100/667 ;Zhlerstand fr Effekt 2
.equ	impcn3=	((effct3+100)*5+1000)*100/667 ;Zhlerstand fr Effekt 3
;
;
; Register-Definitionen
;
.def	timerh=	r2		;High-Byte fr Timer0, wird bei jedem
				;Timer0 Overflow Interrupt um 1 erhht
.def	backup=	r3		;SREG-Zwischenspeicher fr Interrupts
.def	iregl1=	r4		;Arbeitsregister 1 L fr Interrupts
.def	iregh1=	r5		;Arbeitsregister 1 H fr Interrupts
.def	flags=	r23		;verschiedene Flags
				;Bit0: LED-Effekt 1 aktiv
				;Bit1: LED-Effekt 2 aktiv
				;Bit2: LED-Effekt 3 aktiv
				;Bit3: letzter Status vom Empf.-Eingang
.def	iregl2=	r24		;Arbeitsregister 2 L fr Interrupts
.def	iregh2=	r25		;Arbeitsregister 2 H fr Interrupts
;
.cseg
.org	0
;
reset:	rjmp	start		;Programm-Beginn
int0ad:	reti			;External Int 0 (nicht genutzt)
pci0ad:	rjmp	pcicnt		;Pin Change Interrupt, wird bei Pegel-
				;Wechsel am Eingang PB3 ausgelst
ovf0ad:	rjmp	timer0		;Timer0 Overflow, bei laufendem Timer0
				;585,9Hz bzw. alle 1,707ms
erdyad:	reti			;EEPROM Ready (nicht genutzt)
aciadd:	reti			;Analog Comparator (nicht genutzt)
oc0aad:	reti			;Timer0 Compare Match A (nicht genutzt)
oc0bad:	reti			;Timer0 Compare Match B (nicht genutzt)
wdtadd:	reti			;Watchdog Time-out (nicht genutzt)
adccad:	reti			;ADC Conversion Complete (nicht gen.)
;
; Initialisierung ---------------------
;
start:	ldi	r16,0b010111		;alle Ports auer B3 auf Aus-
	out	ddrb,r16		;gang setzen
	ldi	r16,0b111111		;Ausgang auf High und Pull-up
	out	portb,r16		;bei den Eingngen aktivieren
;
	clr	r16
	out	tcnt0,r16		;Timer0 Zhler lschen
	out	tccr0a,r16		;Timer0 im Normal-Modus
	ldi	r16,(1<<cs01)+(1<<cs00)
	out	tccr0b,r16		;Timer 0, Vorteiler=64 setzen
	ldi	r16,1<<toie0		;Overflow Interrupt aktivieren
	out	timsk0,r16		;Interrupt einschalten
	ldi	r16,(1<<tsm)+(1<<psr10)
	out	gtccr,r16		;Vorteiler anhalten/rcksetzen
;
	ldi	r16,1<<isc00		;Pin Change Modus aktivieren
	out	mcucr,r16
	ldi	r16,1<<pcie		;Pin Change Interrupt
	out	gimsk,r16		;aktivieren
	ldi	r16,1<<pcint3		;Pin Change an PB3/PCI3
	out	pcmsk,r16		;aktivieren
;
	clr	timerh			;Timer H-Byte lschen
	clr	flags			;alle Flags lschen
	wdr				;Watchdog-Timer zurcksetzen
	ldi	r16,(1<<wde)+(1<<wdp3)+(1<<wdp0);Watchdog ein, 1024k
	out	wdtcr,r16		;Teiler setzen (Zykluszeit=8s)
	sei				;Interrupts aktivieren
;
	in	r16,pinb		;Pin-Belegung PortB holen
	bst	r16,3			;Status von PB3 holen
	bld	flags,3			;und in den Flags speichern
	brtc	main			;PB3 Low? ja -> Hauptprogramm
	ori	flags,0b00000111	;sonst alle Effekte einschalten
;
;
; Hauptprogramm -----------------------
;
main:	wdr				;Watchdog-Timer zurcksetzen
;
; Blink-Effekte steuern (LED-Effekt 2)
;
	bst	flags,1			;Effekt 2 aktiviert?
	brts	ma0010			;ja -> Effekt steuern
	ldi	r19,1			;sonst Warteschleife 20ms und
	rcall	wait10			;andere Effekte steuern
	rjmp	main			;Schleife
;
ma0010:	ldi	r20,5			;5 Lichtblitze an PB2 ausgeben
ma0020:	sbi	portb,2			;LED an PB2 ausschalten
	bst	flags,1			;Effekt 2 aktiv?
	brtc	ma0030			;nein -> LED an PB2 bleibt aus
	cbi	portb,2			;sonst LED an PB2 einschalten
ma0030:	rcall	wait			;Warteschleife
	sbi	portb,2			;LED an PB2 wieder ausschalten
	rcall	wait			;Warteschleife
	dec	r20			;alle Lichtblitze ausgegeben?
	brne	ma0020			;nein -> Schleife
	rcall	waitx			;lange Warteschleife (Pause)
;
	ldi	r20,5			;5 Lichtblitze an PB4 ausgeben
ma0040:	sbi	portb,4			;LED an PB4 ausschalten
	bst	flags,1			;Effekt 2 aktiv?
	brtc	ma0050			;nein -> LED an PB4 bleibt aus
	cbi	portb,4			;sonst LED an PB4 einschalten
ma0050:	rcall	wait			;Warteschleife
	sbi	portb,4			;LED an PB4 wieder ausschalten
	rcall	wait			;Warteschleife
	dec	r20			;alle Lichtblitze ausgegeben?
	brne	ma0040			;nein -> Schleife
	rcall	waitx			;lange Warteschleife (Pause)
;
	rjmp	main			;Schleife
;
; Warteschleife fr alle Blink-Effekte und Steuerung des Status fr
; LED-Effekt 1 und LED-Effekt 3
;
wait:	ldi	r19,3			;Zeitkonstante laden (3x20ms)
wait10:	clr	r18			;Zeitkonstante mittl. Schleife
	clr	r17			;Zeitkonstante innere Schleife
wait20:	dec	r17			;innere Schleife zhlen
	brne	wait20			;Ende? nein -> Schleife
	dec	r18			;mittlere Schleife zhlen
	brne	wait20			;Ende? nein -> Schleife
;
	bst	flags,0			;Effekt 1 aktiv?
	brts	wait30			;ja -> LED an PB0 einschalten
	sbi	portb,0			;sonst LED an PB0 ausschalten
	rjmp	wait40
wait30:	cbi	portb,0			;LED an PB0 einschalten
;
wait40:	bst	flags,2			;Effekt 3 aktiv?
	brts	wait50			;ja -> LED an PB1 einschalten
	sbi	portb,1			;sonst LED an PB1 ausschalten
	rjmp	wait60
wait50:	cbi	portb,1			;LED an PB1 einschalten
;
wait60:	dec	r19			;uere Schleife zhlen
	brne	wait20			;Ende? nein -> Schleife
	ret
;
; Lange Warteschleife fr Pausen zwischen den Blink-Effekten
;
waitx:	ldi	r19,40			;Zeitkonstante laden (40x20ms)
	rjmp	wait10			;zur normalen Warteschleife
;
;
; Pin Change Interrupt ----------------
;
pcicnt:	in	backup,sreg		;SREG sichern
	in	iregl2,pinb		;Pin-Belegung PortB holen
	andi	iregl2,0b001000		;PB3 filtern
	mov	iregh2,flags		;Flags kopieren
	andi	iregh2,0b00001000	;letzten Status filtern
	eor	iregh2,iregl2		;hat sich der Status gendert?
	breq	intend			;nein -> Ende
;
	bst	iregl2,3		;sonst neuen Status holen
	bld	flags,3			;und in den Flags speichern
	brts	pci100			;Impuls-Beginn? ja -> bearbeit.
;
; Impuls-Ende bearbeiten
;
	ldi	r16,(1<<tsm)+(1<<psr10)
	out	gtccr,r16		;Vorteiler anhalten/rcksetzen
	in	iregl1,tcnt0		;aktuellen Timer0-Stand holen
	mov	iregh1,timerh		;Timer0 High-Byte holen
	mov	iregl2,iregl1		;Zhlerstand L holen und mit H
	or	iregl2,iregh1		;verknpfen, beide=0?
	breq	intend			;ja -> Ende
;
	cbr	flags,0b00000111	;alle Effekte ausschalten
	ldi	iregl2,low(impcn1)
	ldi	iregh2,high(impcn1)	;Zhlerstand fr Effekt 1 holen
	sub	iregl2,iregl1
	sbc	iregh2,iregh1		;Zhlerstand berschritten?
	brcc	pci010			;nein -> weiter
	sbr	flags,0b00000001	;sonst Lichteffekt 1 einschalt.
;
pci010:	ldi	iregl2,low(impcn2)
	ldi	iregh2,high(impcn2)	;Zhlerstand fr Effekt 2 holen
	sub	iregl2,iregl1
	sbc	iregh2,iregh1		;Zhlerstand berschritten?
	brcc	pci020			;nein -> weiter
	sbr	flags,0b00000010	;sonst Lichteffekt 2 einschalt.
;
pci020:	ldi	iregl2,low(impcn3)
	ldi	iregh2,high(impcn3)	;Zhlerstand fr Effekt 3 holen
	sub	iregl2,iregl1
	sbc	iregh2,iregh1		;Zhlerstand berschritten?
	brcc	intend			;nein -> weiter
	sbr	flags,0b00000100	;sonst Lichteffekt 3 einschalt.
	rjmp	intend
;
; Impuls-Beginn bearbeiten
;
pci100:	clr	r16
	out	tcnt0,r16		;Timer0 Zhler lschen
	mov	timerh,r16		;Timer0 H-Byte lschen
	out	gtccr,r16		;Vorteiler starten
	rjmp	intend
;
; Timer0 Overflow Interrupt -----------
;
timer0:	in	backup,sreg		;SREG sichern
	inc	timerh			;Timer0 H-Byte erhhen
	mov	iregl2,timerh		;Timer0 H-Byte kopieren
	cpi	iregl2,18		;H-Byte>18 (ber 30ms)?
	brcs	intend			;nein -> Ende
;
	ldi	r16,(1<<tsm)+(1<<psr10)	;sonst Dauer-High am Eingang:
	out	gtccr,r16		;Vorteiler anhalten/rcksetzen
	sbr	flags,0b00000111	;alle Effekte einschalten
;
intend:	out	sreg,backup		;SREG wiederherstellen
	reti
;
